// -*- C++ -*-
/*!
 * @file  CoordTrans2D.cpp
 * @brief Coordinate transformation of a 2D point using the given translation vector and rotation angle 
 * @date $Date$
 *
 * $Id$
 */

#include "CoordTrans2D.h"

// Module specification
// <rtc-template block="module_spec">
static const char* coordtrans2d_spec[] =
  {
    "implementation_id", "CoordTrans2D",
    "type_name",         "CoordTrans2D",
    "description",       "Coordinate transformation of a 2D point using the given translation vector and rotation angle ",
    "version",           "1.0.0",
    "vendor",            "TekeshiSasaki",
    "category",          "generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

CoordTrans2D::CoordTrans2D(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_XinIn("Xin", m_Xin),
    m_YinIn("Yin", m_Yin),
    m_TxIn("Tx", m_Tx),
    m_TyIn("Ty", m_Ty),
    m_ThetaIn("Theta", m_Theta),
    m_XoutOut("Xout", m_Xout),
    m_YoutOut("Yout", m_Yout),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Xin", m_XinIn);
  registerInPort("Yin", m_YinIn);
  registerInPort("Tx", m_TxIn);
  registerInPort("Ty", m_TyIn);
  registerInPort("Theta", m_ThetaIn);
  
  // Set OutPort buffer
  registerOutPort("Xout", m_XoutOut);
  registerOutPort("Yout", m_YoutOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  tx = 0;
  ty = 0;
  theta = 0;
}

CoordTrans2D::~CoordTrans2D()
{
}


/*
RTC::ReturnCode_t CoordTrans2D::onInitialize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordTrans2D::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordTrans2D::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordTrans2D::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CoordTrans2D::onActivated(RTC::UniqueId ec_id)
{
  m_XinIn.read();
  m_YinIn.read();
  m_TxIn.read();
  m_TyIn.read();
  m_ThetaIn.read();

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t CoordTrans2D::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CoordTrans2D::onExecute(RTC::UniqueId ec_id)
{
  double xin, yin;
  double xout, yout;


  //New transformation parameters
  if(m_TxIn.isNew()){
    m_TxIn.read();
    tx = m_Tx.data;
  }
  if(m_TyIn.isNew()){
    m_TyIn.read();
    ty = m_Ty.data;
  }
  if(m_ThetaIn.isNew()){
    m_ThetaIn.read();
    theta = m_Theta.data;
  }

  //New 2D point
  if(m_XinIn.isNew() && m_YinIn.isNew()){
    m_XinIn.read();
    m_YinIn.read();

    xin = m_Xin.data;
    yin = m_Yin.data;

    //Transformation
    xout = xin * std::cos(theta) - yin * std::sin(theta) + tx;
    yout = xin * std::sin(theta) + yin * std::cos(theta) + ty;

    //Output
    std::cout << "Transformed: (" << xin << ',' << yin << ") -> (" << xout << ',' << yout << ')' << std::endl;
    m_Xout.data = xout;
    m_Yout.data = yout;
    m_XoutOut.write();
    m_YoutOut.write();
  }


  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t CoordTrans2D::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordTrans2D::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordTrans2D::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordTrans2D::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CoordTrans2D::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void CoordTrans2DInit(RTC::Manager* manager)
  {
    RTC::Properties profile(coordtrans2d_spec);
    manager->registerFactory(profile,
                             RTC::Create<CoordTrans2D>,
                             RTC::Delete<CoordTrans2D>);
  }
  
};


